{% if FEINCMS_ADMIN_MEDIA_HOTLINKING %}
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.5.1/jquery.min.js"></script>
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.7/jquery-ui.min.js"></script>
{% else %}
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery-1.5.1.min.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery-ui-1.8.7.custom.min.js"></script>
{% endif %}
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery.color.js"></script>
<script type="text/javascript">
    var feincms = {
        {% if FEINCMS_JQUERY_NO_CONFLICT %}
        "jQuery": jQuery.noConflict(true)
        {% else %}
        "jQuery": jQuery
        {% endif %}
    };
</script>

<div id="{{ identifier }}-new">
    {{ content|safe }}
</div>

<script type="text/javascript">
opener.feincms.fe_update_content('{{ identifier }}', feincms.jQuery('#{{ identifier }}-new').html());

window.close();
</script>
