{% extends "admin/change_list.html" %}
{% load adminmedia admin_list i18n %}

{% block extrahead %}
{{ block.super }}
<link rel="stylesheet" type="text/css" href="{{ FEINCMS_ADMIN_MEDIA }}style.css" />
<link rel="stylesheet" type="text/css" href="{{ FEINCMS_ADMIN_MEDIA }}fein_tree.css" />

{% if FEINCMS_ADMIN_MEDIA_HOTLINKING %}
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.5.1/jquery.min.js"></script>
<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.7/jquery-ui.min.js"></script>
{% else %}
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery-1.5.1.min.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery-ui-1.8.7.custom.min.js"></script>
{% endif %}
<script type="text/javascript">
    var feincms = {
        {% if FEINCMS_JQUERY_NO_CONFLICT %}
            "jQuery": jQuery.noConflict(true)
        {% else %}
            "jQuery": jQuery
        {% endif %}
        , "tree_structure": {{ tree_structure|default:"null" }}
    };
</script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}ie_compat.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}jquery.cookie.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}toolbox.js"></script>
<script type="text/javascript" src="{{ FEINCMS_ADMIN_MEDIA }}fein_tree.js"></script>

{% endblock %}


{% block filters %}
	{% if cl.has_filters %}
		<div id="changelist-filter">
			<h2>{% trans 'Shortcuts' %}</h2>
			<ul>
				<li><a id="collapse_entire_tree" href="#">{% trans 'Collapse tree' %}</a></li>
				<li><a id="open_entire_tree" href="#">{% trans 'Expand tree' %}</a></li>
			</ul>

			<h2>{% trans 'Filter' %}</h2>
			{% for spec in cl.filter_specs %}{% admin_list_filter cl spec %}{% endfor %}
		</div>
	{% endif %}
{% endblock %}
