{% extends "admin/feincms/item_editor.html" %}

{% load i18n %}

{% block object-tools %}
{{ block.super }}
<ul class="extra-object-tools">
	{% if original.is_active and FEINCMS_FRONTEND_EDITING %}
	    <li><a target="_blank" href="{{ original.get_absolute_url }}?frontend_editing=1" class="frontend_editing_link">{% trans "Edit on site" %}</a></li>
	{% endif %}
</ul>
{% endblock %}

{% block breadcrumbs %}
<div class="breadcrumbs">
     <a href="../../../">{% trans "Home" %}</a> &rsaquo;
     <a href="../">{{ opts.verbose_name_plural|capfirst }}</a> &rsaquo;

     {% for p in original.get_ancestors %}
       <a href="../{{ p.id }}/">{{ p.short_title }}</a> :
     {% endfor %}
     {{ original.short_title }}
</div>
{% endblock %}
